/*********************************************************
*
*  Multi Theft Auto: San Andreas - Deathmatch
*
*  MySQL 5.0, External lua add-on module
*  http://www.multitheftauto.com
*
*  Copyright  2003-2008 MTA.  All Rights Reserved.
*
*  Grand Theft Auto is  2002-2003 Rockstar North
*
*  THE FOLLOWING SOURCES ARE PART OF THE MULTI THEFT
*  AUTO SOFTWARE DEVELOPMENT KIT AND ARE RELEASED AS
*  OPEN SOURCE FILES. THESE FILES MAY BE USED AS LONG
*  AS THE DEVELOPER AGREES TO THE LICENSE THAT IS
*  PROVIDED WITH THIS PACKAGE.
*
*********************************************************/

// CFunctions implements our lua functionality in C

#include "CFunctions.h"

CFunctions::CFunctions ( void )
{

}

CFunctions::~CFunctions ( void )
{

}

int CFunctions::MySQLCreate ( lua_State* luaVM )
{
	if ( luaVM )
	{
		CMySQL *pMySQLThread = new CMySQL;
		lua_pushlightuserdata ( luaVM, pMySQLThread );
		return 1;
	}
	lua_pushboolean ( luaVM, false );
	return 0;
}

int CFunctions::MySQLOpen ( lua_State* luaVM )
{
	if ( luaVM )
	{
		if ( lua_type ( luaVM, 1 ) == LUA_TLIGHTUSERDATA )
		{
			CMySQL *pMySQLThread = (CMySQL*) lua_touserdata ( luaVM, 1 );

			if ( pMySQLThread )
			{
				// create a new thread
				CMySQLData *pMySQLData = new CMySQLData;

				// no need to lock the logical mutex (it's not running yet)
				pMySQLData->iCommand = CMySQLData::MYSQL_OPEN;
				pMySQLData->nParameters = 5;
				pMySQLData->pParameter[0] = (void*)lua_tostring ( luaVM, 3 );
				pMySQLData->pParameter[1] = (void*)lua_tostring ( luaVM, 4 );
				pMySQLData->pParameter[2] = (void*)lua_tostring ( luaVM, 5 );
				pMySQLData->pParameter[3] = (void*)lua_tostring ( luaVM, 6 );
				pMySQLData->pParameter[4] = (void*)(int)(lua_tonumber ( luaVM, 7 ));

				// set up the callback function
				pMySQLData->szCallbackFunction = lua_tostring ( luaVM, 2 );
				pMySQLData->luaVM = luaVM;

				// start the thread
				if ( pMySQLThread->Start ( pMySQLData ) == false )
					pModuleManager->Printf ( "MODULE: Critical error. Could not spawn a new thread!\n" );

				CThread::Unlock ( &pMySQLData->MutexLogical );
				return 1;
			}
		}
	}
	return 0;
}

int CFunctions::MySQLDestroy ( lua_State* luaVM )
{
	if ( luaVM )
	{
		if ( lua_type ( luaVM, 1 ) == LUA_TLIGHTUSERDATA )
		{
			pModuleManager->Printf ( "MODULE: Closing MySQL connection thread.\n" );

			CMySQL *pMySQLThread = (CMySQL*) lua_touserdata ( luaVM, 1 );
			if ( pMySQLThread && pMySQLThread->pData )
			{
				delete pMySQLThread;
				return 1;
			}

		}
	}
	return 0;
}

int CFunctions::MySQLQuery ( lua_State* luaVM )
{
	if ( luaVM )
	{
		if ( lua_type ( luaVM, 1 ) == LUA_TLIGHTUSERDATA )
		{
			CMySQL *pMySQLThread = (CMySQL*) lua_touserdata ( luaVM, 1 );
			if ( pMySQLThread && pMySQLThread->pData )
			{
				// This fails whenever the thread's busy
				if ( CThread::TryLock ( &pMySQLThread->pData->MutexLogical ) )
				{
					// set parameters
					pMySQLThread->pData->iCommand = CMySQLData::MYSQL_QUERY;
					pMySQLThread->pData->nParameters = 1;
					pMySQLThread->pData->pParameter[0] = (void*)lua_tostring ( luaVM, 3 );

					// set up the callback function
					pMySQLThread->pData->szCallbackFunction = lua_tostring ( luaVM, 2 );
					pMySQLThread->pData->luaVM = luaVM;

					CThread::Unlock ( &pMySQLThread->pData->MutexLogical );
					return 1;
				}
			}
		}
	}
	return 0;
}

int CFunctions::MySQLSafeString ( lua_State* luaVM )
{
	if ( luaVM )
	{
		if ( lua_type ( luaVM, 1 ) == LUA_TLIGHTUSERDATA )
		{
			CMySQL *pMySQLThread = (CMySQL*) lua_touserdata ( luaVM, 1 );
			
			// pData is only valid when a mysql thread was started
			// (mysql_real_escape_string works only on mysql classes with established connections)
			if ( pMySQLThread && pMySQLThread->pData )
			{
				// according to the mysql c api documentation, we should allocate a string of at least (length*2+1) bytes
				const char *szQuery = lua_tostring ( luaVM, 2 );\
				char *szBuffer = new char [ strlen ( szQuery ) * 2 + 1 ];

				// escape the string
				mysql_real_escape_string ( &(pMySQLThread->pDB), szBuffer, szQuery, strlen ( szQuery ) );

				// push the lua data
				lua_pushstring ( luaVM, szBuffer );

				delete [] szBuffer;
				return 1;
			}
		}
	}
	lua_pushboolean ( luaVM, false );
	return 0;
}
