/*********************************************************
*
*  Multi Theft Auto: San Andreas - Deathmatch
*
*  MySQL 5.0, External lua add-on module
*  http://www.multitheftauto.com
*
*  Copyright  2003-2008 MTA.  All Rights Reserved.
*
*  Grand Theft Auto is  2002-2003 Rockstar North
*
*  THE FOLLOWING SOURCES ARE PART OF THE MULTI THEFT
*  AUTO SOFTWARE DEVELOPMENT KIT AND ARE RELEASED AS
*  OPEN SOURCE FILES. THESE FILES MAY BE USED AS LONG
*  AS THE DEVELOPER AGREES TO THE LICENSE THAT IS
*  PROVIDED WITH THIS PACKAGE.
*
*********************************************************/

// CMySQL manages the threaded execution of queries

#include "CMySQL.h"

CMySQL::CMySQL ( void )
{
	pData = NULL;
}

CMySQL::~CMySQL ( void )
{
}

int CMySQL::Execute ( CThreadData *pIn )
{
	pModuleManager->Printf ( "MODULE: New MySQL thread successfully created.\n" );

	bool bRetval = false;

	// get the thread data that was passed by the calling function
	pData = reinterpret_cast < CMySQLData* > ( pIn );

	// initialize the mysql handler
	mysql_init ( &pDB );

	// lock the kill mutex
	CThread::Lock ( &(pData->MutexKill) );

	// start the main loop (ends whenever bAbortThread is true)
	while ( !pData->bAbortThread )
	{
		switch ( pData->iCommand )
		{
			case CMySQLData::MYSQL_OPEN:	// (host, user, password, database, port)
				CThread::Lock ( &(pData->MutexLogical) );
				pData->iCommand = 0;
				if ( pData->nParameters == 5 )
				{
					// connect to the mysql database
					if ( mysql_real_connect ( &pDB, (const char*)pData->pParameter[0], (const char*)pData->pParameter[1], (const char*)pData->pParameter[2], (const char*)pData->pParameter[3], (int)pData->pParameter[4], NULL, 0 ) )
					{
						bRetval = true;
					}

					// clean up
					pData->nParameters = 0;

					// call the lua function
					lua_getglobal ( pData->luaVM, pData->szCallbackFunction );
					//lua_pushlightuserdata ( pData->luaVM, this );
					lua_pushboolean ( pData->luaVM, bRetval );
					lua_pcall ( pData->luaVM, 1, 0, 0 );
				}
				CThread::Unlock ( &(pData->MutexLogical) );
				break;
			case CMySQLData::MYSQL_QUERY:
				CThread::Lock ( &(pData->MutexLogical) );
				pData->iCommand = 0;
				if ( pData->nParameters == 1 )
				{
					MYSQL_RES *result = NULL;
					MYSQL_ROW row;
					const char *szQuery = (const char*)pData->pParameter[0];
					unsigned int num_rows = 0, num_fields = 0;

					// execute the query
					if ( mysql_real_query ( &pDB, szQuery, strlen ( szQuery ) ) == 0 )
					{
						lua_getglobal ( pData->luaVM, pData->szCallbackFunction );
						if ( mysql_field_count ( &pDB ) > 0 )
						{
							// get the results
							result = mysql_store_result ( &pDB );
							num_rows = mysql_num_rows ( result );
							num_fields = mysql_num_fields ( result );

							if ( result && num_rows > 0 )
							{
								// store our result in a table structure using some low-level lua table stack code
								lua_newtable ( pData->luaVM );
								for ( unsigned int i = 0; i < num_rows; i++ ) {
									row = mysql_fetch_row ( result );
									lua_newtable ( pData->luaVM );
									lua_pushnumber ( pData->luaVM, i+1 );
									lua_pushvalue ( pData->luaVM, -2 );
									lua_settable ( pData->luaVM, -4 );
									for ( unsigned int j = 0; j < num_fields; j++ ) {
										lua_pushnumber ( pData->luaVM, j+1 );
										if ( row[j] != NULL )
											lua_pushstring ( pData->luaVM, row[j] );
										else
											lua_pushnil ( pData->luaVM );
										lua_settable ( pData->luaVM, -3 );
									}
									lua_pop ( pData->luaVM, 1 );

									// free the result
									mysql_free_result ( result );
								}
							} else {
								lua_pushboolean ( pData->luaVM, true );
							}
						} else {
							// queries without any result rows
							lua_pushboolean ( pData->luaVM, true );
						}
						// call the lua function
						lua_pcall ( pData->luaVM, 1, 0, 0 );

						// query succeeded
						mysql_free_result ( result );
						pData->nParameters = 0;
						CThread::Unlock ( &(pData->MutexLogical) );
						break;
					} else {
						// query error
						pModuleManager->Printf ( "ERROR: Query failed: %s\n", mysql_error ( &pDB ) );
						pModuleManager->DebugPrintf ( pData->luaVM, "Database query failed: %s", mysql_error ( &pDB ) );
						lua_pushboolean ( pData->luaVM, false );
						lua_pcall ( pData->luaVM, 1, 0, 0 );
					}
					pData->nParameters = 0;
				}
				CThread::Unlock ( &(pData->MutexLogical) );
				break;
			default:
				break;
		}

		// time cycles to save unnecessary processing time
		Sleep ( 500 );
	}
	// clean up mysql
	mysql_close ( &pDB );

	pModuleManager->Printf ( "MODULE: MySQL thread gracefully closed.\n" );

	// unlock the kill mutex, so the function that requested a graceful thread shutdown can continue
	CThread::Unlock ( &(pData->MutexKill) );
	return true;
}
