/*********************************************************
*
*  Multi Theft Auto: San Andreas - Deathmatch
*
*  MySQL 5.0, External lua add-on module
*  http://www.multitheftauto.com
*
*  Copyright  2003-2008 MTA.  All Rights Reserved.
*
*  Grand Theft Auto is  2002-2003 Rockstar North
*
*  THE FOLLOWING SOURCES ARE PART OF THE MULTI THEFT
*  AUTO SOFTWARE DEVELOPMENT KIT AND ARE RELEASED AS
*  OPEN SOURCE FILES. THESE FILES MAY BE USED AS LONG
*  AS THE DEVELOPER AGREES TO THE LICENSE THAT IS
*  PROVIDED WITH THIS PACKAGE.
*
*********************************************************/

// CThread is the threading base class

#include "CThread.h"

CThread::CThread ( void )
{
	m_pThreadData = NULL;
    m_pArg = NULL;
#ifdef WIN32	// win32 threads
    m_hThread = NULL;
#endif
}


CThread::~CThread ( void )
{
    Stop ();
}


bool CThread::Start ( CThreadData *pData )
{
	if ( pData == NULL )	// pdata has to be valid
		return false;

    Stop ();
    Arg ( pData );

	#ifdef WIN32	// Win32 threads
		m_hThread = CreateThread ( NULL, 0, &CThread::EntryPoint, this, 0, NULL );
		return m_hThread != NULL;
	#else			// POSIX threads
		if ( pthread_create ( &m_hThread, NULL, CThread::EntryPoint, this ) )
		{
			// something bad has happened
			return false;
		}
	#endif
	return true;
}


void CThread::Stop ( void )
{
    if ( m_hThread && m_pThreadData )
    {
		// graceful shutdowns only, to prevent memory leakage
		m_pThreadData->bAbortThread = true;

		// block until the thread has been shut down gracefully (graceful shutdown unlocks the mutex)
		CThread::Lock ( &(m_pThreadData->MutexKill) );
		CThread::Unlock ( &(m_pThreadData->MutexKill) );

		// clean up the thread data
		delete m_pThreadData;
	}
}


bool CThread::TryLock ( ThreadMutex * Mutex )
{
	#ifdef WIN32
		if ( TryEnterCriticalSection ( Mutex ) != 0 )
			return true;
	#else
		if ( pthread_mutex_trylock ( Mutex ) == 0 )
			return true;
	#endif
	return false;
}


void CThread::Lock ( ThreadMutex * Mutex )
{
	#ifdef WIN32	// win32 threads
		EnterCriticalSection ( Mutex );
	#else			// posix threads
		pthread_mutex_lock ( Mutex );
	#endif
}


void CThread::Unlock ( ThreadMutex * Mutex )
{
	#ifdef WIN32	// win32 threads
		LeaveCriticalSection ( Mutex );
	#else			// posix threads
		pthread_mutex_unlock ( Mutex );
	#endif
}


int CThread::Run ( CThreadData* arg )
{
    return Execute ( arg );
}


#ifdef WIN32	// win32 threads
DWORD CThread::EntryPoint ( void* pThis )
{
	CThread* pt = (CThread*)pThis;
	return pt->Run ( pt->Arg () );
}
#else			// posix threads
void* CThread::EntryPoint ( void* pThis )
{
	CThread* pt = (CThread*)pThis;
	pt->Run ( pt->Arg () );
	return NULL;
}
#endif


CThreadData* CThread::Arg ( void ) const
{
    return m_pThreadData;
}


void CThread::Arg ( CThreadData* pData )
{
    m_pThreadData = pData;
}
